import os
import numpy as np
import argparse

import params
import utils

aj_over_ak=3.02  # Nishiyama+06 (ApJ, 638, 839)

def print_spec(waves, fluxs, header=False):
    if header is True:
        print('wavelength,flux')
    for w, f in zip(waves,fluxs):
        print('{0:.2f},{1:.4e}'.format(w,f))


parser=argparse.ArgumentParser(description='load_template.py')
parser.add_argument('-m','--mode',type=str,default='WIDE',help='Mode (WIDE, HIRES-Y, HIRES-J)')
parser.add_argument('-t','--teff',type=float,default=params.option_defaults['teff'],help='Effective temperature (K)')
parser.add_argument('-j','--jmag',type=float,default=params.option_defaults['jmag'],help='J-band magnitude')
parser.add_argument('-s','--step',type=float,default=params.option_defaults['step'],help='Wavelength step (AA)')
parser.add_argument('-ak','--extinction_k',type=float,default=params.option_defaults['ak'],help='K-band extinction (mag)')
parser.add_argument('-aj','--extinction_j',type=float,default=params.option_defaults['aj'],help='J-band extinction (mag)')
parser.add_argument(params.option_short['header'],'--header',default=params.option_defaults['header'],action='store_true',help=params.option_helps['header'])

args=parser.parse_args()

mode=args.mode
if mode not in params.list_mode:
    print('Unexpected mode={0}'.format(mode))
    exit()
wmin, wmax=params.list_wranges[mode]
if args.step:
    step=args.step
if np.isnan(step):
    print('No wavelength step was given')
    exit()
waves=np.arange(wmin,wmax,step)

if args.teff:
    teff=args.teff
if np.isnan(teff):
    print('No Teff was given')
    exit()

jmag=args.jmag
if np.isnan(jmag):
    print('No magnitude inputs!')
    exit()
wave_zero=12500.
flux_zero=3.31E-3*pow(10,-0.4*jmag)

aj=args.extinction_j
if (np.isnan(aj)) and (not np.isnan(args.extinction_k)):
    aj=args.extinction_k*aj_over_ak

fluxs=utils.calc_BB(teff,waves,normalize={'wave':wave_zero,'flux':flux_zero})
if (not np.isnan(aj)) and (aj>0):
    fluxs=utils.apply_extinction(waves,fluxs,aj)
if args.header:
    print_spec(waves,fluxs,header=True)
else:
    print_spec(waves,fluxs,header=False)

