from __future__ import print_function
import os
import sys
import numpy as np
import argparse

import params
import utils

list_arg=['mode','slit','season','seeing','mag','nod_pattern','frame_itime','nframe']
list_option=['verbose','nyquist','cor_factor','telescope','spectrum','ymag','jmag','header']

parser=argparse.ArgumentParser(description='calc_SN.py')
parser.add_argument('mode',type=str,help=params.arg_helps['mode'])
parser.add_argument('slit',type=str,help=params.arg_helps['slit'])
parser.add_argument('season',type=str,help=params.arg_helps['season'])
parser.add_argument('seeing',type=float,help=params.arg_helps['seeing'])
parser.add_argument('nod_pattern',type=str,help=params.arg_helps['nod_pattern'])
parser.add_argument('frame_itime',type=float,help=params.arg_helps['frame_itime'])
parser.add_argument('nframe',type=int,help=params.arg_helps['nframe'])
for option in ['verbose','nyquist','spectrum']:
    parser.add_argument(params.option_short[option],'--'+option,action='store_true',default=params.option_defaults[option],help=params.option_helps[option])
parser.add_argument(params.option_short['cor_factor'],'--cor_factor',type=float,default=params.option_defaults['cor_factor'],help=params.option_helps['cor_factor'])
parser.add_argument(params.option_short['telescope'],'--telescope',type=str,default=params.option_defaults['telescope'],help=params.option_helps['telescope'])
parser.add_argument(params.option_short['ymag'],'--ymag',type=float,default=params.option_defaults['ymag'],help=params.option_helps['ymag'])
parser.add_argument(params.option_short['jmag'],'--jmag',type=float,default=params.option_defaults['jmag'],help=params.option_helps['jmag'])
parser.add_argument(params.option_short['header'],'--header',action='store_true',default=params.option_defaults['header'],help=params.option_helps['header'])

setting=params.init_setting()

args=parser.parse_args()

if args.verbose:
    setting['verbose']=args.verbose
if args.nyquist:
    setting['nyquist']=args.nyquist
if args.cor_factor:
    setting['cor_factor']=args.cor_factor
if args.telescope:
    setting['telescope']=args.telescope
if args.ymag:
    setting['ymag']=args.ymag
if args.jmag:
    setting['jmag']=args.jmag
if args.spectrum:
    setting['spectrum']=args.spectrum

setting['mode']        =args.mode
setting['slit']        =args.slit
setting['season']      =args.season
setting['seeing']      =args.seeing
setting['nod_pattern'] =args.nod_pattern
setting['frame_itime'] =args.frame_itime
setting['nframe']      =args.nframe

tel_params=params.get_telescope_params(setting['telescope'])
if tel_params is None:
    exit()

if setting['verbose'] is True:
    print('### Parameters validation and calculation ###',file=sys.stderr)

if params.validate_setting(setting, tel_params) is not True:
    print('!!validate_setting failed!!',file=sys.stderr)
    print(setting,file=sys.stderr)
    exit()

setting['nod_sequence']=params.make_nod_sequence(setting['nod_pattern'],setting['nframe'],verbose=setting['verbose'])
ntarget, nsky, n_nod=params.count_nod_numbers(setting['nod_sequence'],verbose=setting['verbose'])
setting['ntarget']=ntarget
setting['nsky']=nsky
setting['n_nod']=n_nod

wmin, wmax=params.list_wranges[setting['mode']]
spec=[]
if (not np.isnan(setting['ymag'])) and ((wmin-10305.)*(wmax-10305.)<=0):
    spec.append([10305.,5.71E-3*pow(10,-0.4*setting['ymag'])])
if (not np.isnan(setting['jmag'])) and ((wmin-12150.)*(wmax-12150.)<=0):
    spec.append([12150.,3.31E-3*pow(10,-0.4*setting['jmag'])])
if setting['spectrum'] is True:
    for line in sys.stdin.readlines():
        if 'wave' in line:
            continue
        if ',' in line:
            items=line.split(',')
        else:
            items=line.split()
        w, f=float(items[0]), float(items[1])
        if (w>=wmin) and (w<=wmax) and (f>0):
            spec.append([float(items[0]), float(items[1])])
if (setting['mode']=='HIRES-Y') and (np.isnan(setting['ymag'])) and (not np.isnan(setting['jmag'])):
    spec.append([10305.,5.71E-3*pow(10,-0.4*setting['jmag'])])
    if setting['verbose']:
        print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!',file=sys.stderr)
        print('Only Jmag given to HIRES-Y',file=sys.stderr)
        print('Ymag is assumed to be same as Jmag={0:.3f}'.format(setting['jmag']),file=sys.stderr)
        print('Giving Ymag or Spec,Flux is recommended.',file=sys.stderr)
        print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!',file=sys.stderr)
if len(spec)<=0:
    print('No appropriate brightness information was given',file=sys.stderr)
    exit()
#print(spec)

if setting['verbose'] is True:
    print('### S/N calculation ###',file=sys.stderr)

total_itime=setting['frame_itime']*len(setting['nod_sequence'])
ntarget, nsky, n_nod=params.count_nod_numbers(setting['nod_sequence'],verbose=False)
list_SN=utils.calc_SN(setting,tel_params,setting['frame_itime'],setting['nod_sequence'],spec)
if args.header:
    #print('wavelength,flux,SN')
    print('wavelength,flux,SN,frame_sec,nframe,nod_sequence')
for (wave_AA, flux), SN in zip(spec, list_SN):
    #print('{0:.3f},{1:.3e},{2:.2f}'.format(wave,flux,SN))
    print('{0:.2f},{1:.3e},{2:.0f},{3:.0f},{4:d},{5}'.format(wave_AA,flux,SN,setting['frame_itime'],(ntarget+nsky),setting['nod_sequence']))

