from __future__ import print_function
import os
import sys
import numpy as np
import argparse

import params
import utils

list_arg=['mode','slit','season','seeing','nod_pattern','SN']
list_option=['verbose','nyquist','cor_factor','telescope','max_itime_total','ymag','jmag','max_itime','spectrum']

parser=argparse.ArgumentParser(description='calc_obstime.py')
parser.add_argument('mode',type=str,help=params.arg_helps['mode'])
parser.add_argument('slit',type=str,help=params.arg_helps['slit'])
parser.add_argument('season',type=str,help=params.arg_helps['season'])
parser.add_argument('seeing',type=float,help=params.arg_helps['seeing'])
parser.add_argument('nod_pattern',type=str,help=params.arg_helps['nod_pattern'])
parser.add_argument('SN',type=float,help=params.arg_helps['SN'])

for option in ['verbose','nyquist','spectrum']:
    parser.add_argument(params.option_short[option],'--'+option,action='store_true',default=params.option_defaults[option],help=params.option_helps[option])
parser.add_argument(params.option_short['cor_factor'],'--cor_factor',type=float,default=params.option_defaults['cor_factor'],help=params.option_helps['cor_factor'])
parser.add_argument(params.option_short['telescope'],'--telescope',type=str,default=params.option_defaults['telescope'],help=params.option_helps['telescope'])
parser.add_argument(params.option_short['ymag'],'--ymag',type=float,default=params.option_defaults['ymag'],help=params.option_helps['ymag'])
parser.add_argument(params.option_short['jmag'],'--jmag',type=float,default=params.option_defaults['jmag'],help=params.option_helps['jmag'])
parser.add_argument(params.option_short['max_itime'],'--max_itime',type=float,default=params.option_defaults['max_itime'],help=params.option_helps['max_itime'])
parser.add_argument(params.option_short['max_itime_total'],'--max_itime_total',type=float,default=params.option_defaults['max_itime_total'],help=params.option_helps['max_itime_total'])
parser.add_argument(params.option_short['header'],'--header',action='store_true',default=params.option_defaults['header'],help=params.option_helps['header'])

setting=params.init_setting()

args=parser.parse_args()

if args.verbose:
    setting['verbose']=args.verbose
if args.nyquist:
    setting['nyquist']=args.nyquist
if args.cor_factor:
    setting['cor_factor']=args.cor_factor
if args.telescope:
    setting['telescope']=args.telescope
if args.max_itime_total:
    setting['max_itime_total']=args.max_itime_total
if args.max_itime:
    setting['max_itime']=args.max_itime
if args.ymag:
    setting['ymag']=args.ymag
if args.jmag:
    setting['jmag']=args.jmag
if args.spectrum:
    setting['spectrum']=args.spectrum

setting['mode']        =args.mode
setting['slit']        =args.slit
setting['season']      =args.season
setting['seeing']      =args.seeing
setting['nod_pattern'] =args.nod_pattern
setting['SN']          =args.SN

tel_params=params.get_telescope_params(setting['telescope'])
if tel_params is None:
    exit()


if setting['verbose'] is True:
    print('### Parameters validation and calculation ###',file=sys.stderr)

if params.validate_setting(setting, tel_params) is not True:
    print('!!validate_setting failed!!',file=sys.stderr)
    print(setting,file=sys.stderr)
    exit()

#print(setting,file=sys.stderr)

wmin, wmax=params.list_wranges[setting['mode']]
wmin, wmax=params.list_wranges[setting['mode']]
spec=[]
if (not np.isnan(setting['ymag'])) and ((wmin-10305.)*(wmax-10305.)<=0):
    spec.append([10305.,5.71E-3*pow(10,-0.4*setting['ymag'])])
if (not np.isnan(setting['jmag'])) and ((wmin-12150.)*(wmax-12150.)<=0):
    spec.append([12150.,3.31E-3*pow(10,-0.4*setting['jmag'])])
if setting['spectrum'] is True:
    for line in sys.stdin.readlines():
        if 'wave' in line:
            continue
        if ',' in line:
            items=line.split(',')
        else:
            items=line.split()
        w, f=float(items[0]), float(items[1])
        if (w>=wmin) and (w<=wmax) and (f>0):
            spec.append([float(items[0]), float(items[1])])
if (setting['mode']=='HIRES-Y') and (np.isnan(setting['ymag'])) and (not np.isnan(setting['jmag'])):
    spec.append([10305.,5.71E-3*pow(10,-0.4*setting['jmag'])])
    if setting['verbose']:
        print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!',file=sys.stderr)
        print('Only Jmag given to HIRES-Y',file=sys.stderr)
        print('Ymag is assumed to be same as Jmag={0:.3f}'.format(setting['jmag']),file=sys.stderr)
        print('Giving Ymag or Spec,Flux is recommended.',file=sys.stderr)
        print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!',file=sys.stderr)
if len(spec)<=0:
    print('No appropriate brightness information was given',file=sys.stderr)
    exit()
#print(spec,file=sys.stderr)

if setting['verbose'] is True:
    print('### calc_SN calculation ###',file=sys.stderr)

if args.header:
    print('wavelength,flux,SN,obstime_sec,frame_sec,nframe,nod_sequence')
for i, (wave_AA, flux) in enumerate(spec):
    frame_itime, nod_sequence=utils.search_itime_and_sequence(setting,tel_params,wave_AA,flux,setting['SN'],max_itime=setting['max_itime'])
    SN=(utils.calc_SN(setting,tel_params,frame_itime,nod_sequence,[[wave_AA,flux]],silence=False))[0]
    total_itime=frame_itime*len(nod_sequence)
    ntarget, nsky, n_nod=params.count_nod_numbers(nod_sequence,verbose=False)
    overhead=utils.calc_overhead(len(nod_sequence),n_nod,verbose=setting['verbose'])
    total_sec=(total_itime+overhead)
    total_min=(total_itime+overhead)/60.
    print('{0:.2f},{1:.3e},{2:.0f},{3:.0f},{4:.0f},{5:d},{6}'.format(wave_AA,flux,SN,total_sec,frame_itime,(ntarget+nsky),nod_sequence))

