import numpy as np
import matplotlib.pyplot as plt
import subprocess


def calc_exptime(telescope, mode, slit, seeing, SN, band, mag, max_itime=36000.):
    cmd='python /mnt/SharedDisk/WINERED/WINERED_ETC/calc_obstime.py -t {0} {1} {2} winter {3:.2f} ABBA {4:.2f} -{5} {6:.4f} -xi {7:.0f}'.format(telescope,mode,slit,seeing,SN,band,mag,max_itime)
    result=subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, text=True)
    wave_str, flux_str, SN_final, obstime, itime, nframe, nod_sequence=result.stdout.split(',')
    exptime=float(itime)*float(nframe)
    return exptime


hr_max=5.
mag_min, mag_max=7., 17.
slit, seeing, SN='100', 0.6, 100.

for mode in ['WIDE', 'HIRES-Y', 'HIRES-J']:
    for band, band_label in zip(['y','j'],['Y','J']):
        if (mode=='HIRES-Y') and (band=='j'):
            continue
        if (mode=='HIRES-J') and (band=='y'):
            continue
        print(mode,band_label)
        plot_out='Exptime_{0}_{1}mag.png'.format(mode,band_label)
        fig=plt.figure(figsize=(7,4))
        ax=fig.add_subplot(111)
        for color, telescope in zip(['red','deepskyblue','green'],['Magellan', 'NTT', 'Araki']):
            curve_x=[]; curve_y=[]
            mag, step=mag_min, 0.5
            exptime_prev=np.nan
            while mag<mag_max:
                exptime_hr=calc_exptime(telescope, mode, slit, seeing, SN, band, mag)/3600.
                if exptime_hr<0.01:
                    mag+=step
                    continue
                curve_x.append(mag)
                curve_y.append(exptime_hr)
                if exptime_hr>hr_max:
                    break
                if (not np.isnan(exptime_prev)) and (exptime_hr-exptime_prev>0.1) and (step>0.02):
                    step/=2.
                mag+=step
                exptime_prev=exptime_hr
            ax.plot(curve_x,curve_y,color=color,label=telescope)
        ax.set_xlim([mag_min, mag_max])
        ax.set_ylim([0, hr_max])
        ax.set_xlabel('{0}-band magnitude'.format(band_label))
        ax.set_ylabel('Total exposure time (hour)')
        ax.set_title('Exposure time for S/N={0:.0f} ({1})'.format(SN,mode,band_label))
        ax.tick_params(direction='in',length=8)
        ax.legend(bbox_to_anchor=(0.03,0.95),loc='upper left')
        fig.savefig(plot_out)
        plt.close(fig)

