import numpy as np
import matplotlib.pyplot as plt

import utils
templates_dir='templates'

list_template=['HD_147550','BD-01_1792','LTT_2544']
label_template={'HD_147550':'HD 147550 (B9V)','BD-01_1792':'BD-01d1792 (G6V)','LTT_2544':'LTT 2544 (M2)'}
Jmag_template={'HD_147550':6.001,'BD-01_1792':7.621,'LTT_2544':10.013}
list_teff=[10000., 6000., 3000.]
list_aj=[0.0, 2.0, 5.0]

jmag=0.
wave_zero, flux_zero=12500., 3.31E-3

wmin,wmax=9000.,13500.
waves=np.arange(9000., 13501., 100)

plot_out='SED_BBs.png'
fig=plt.figure(figsize=(7,4.5))
ax=fig.add_subplot(111)
for teff, color in zip(list_teff,['blue','green','red']): 
    fluxs=utils.calc_BB(teff,waves,normalize={'wave':wave_zero,'flux':flux_zero})
    ax.plot(waves,fluxs,color=color,label='Teff={0:.0f}K'.format(teff))
ax.legend(bbox_to_anchor=(0.98,0.98),loc='upper right')
ax.set_yscale('log')
ax.set_xlabel('Wavelgnth (AA)')
ax.set_ylabel('Flux (W/m^3)')
ax.set_title('Blackbody (Jmag={0:.2f})'.format(jmag))
ax.set_xlim([wmin,wmax])
ax.grid()
fig.tight_layout()
fig.savefig(plot_out,dpi=200,pad_inches=0.02)
plt.close(fig)

plot_out='SED_AJs.png'
teff=10000.
fig=plt.figure(figsize=(7,4.5))
ax=fig.add_subplot(111)
for aj, color in zip(list_aj,['blue','green','red']): 
    fluxs=utils.calc_BB(teff,waves,normalize={'wave':wave_zero,'flux':flux_zero})
    fluxs_new=utils.apply_extinction(waves,fluxs,aj)

    ax.plot(waves,fluxs_new,color=color,label='A(J)={0:.0f}'.format(aj))
ax.legend(bbox_to_anchor=(0.98,0.02),loc='lower right')
ax.set_yscale('log')
ax.set_xlabel('Wavelgnth (AA)')
ax.set_ylabel('Flux (W/m^3)')
ax.set_title('Blackbody (Teff={0:.0f}, Jmag={1:.2f})'.format(teff,jmag))
ax.set_xlim([wmin,wmax])
ax.grid()
fig.tight_layout()
fig.savefig(plot_out,dpi=200,pad_inches=0.02)
plt.close(fig)

plot_out='SED_templates.png'
fig=plt.figure(figsize=(7,4.5))
ax=fig.add_subplot(111)
for (name, label), color in zip(label_template.items(),['blue','green','red']):
    file_in='{0}/XSL_{1}.txt'.format(templates_dir,name)
    flux_factor=np.power(10.,0.4*Jmag_template[name])
    waves, fluxs=utils.load_template(file_in,wmin,wmax,flux_factor=flux_factor)
    ax.plot(waves,fluxs,color=color,label=label)
ax.legend(bbox_to_anchor=(0.98,0.98),loc='upper right')
ax.set_yscale('log')
ax.set_xlabel('Wavelgnth (AA)')
ax.set_ylabel('Flux (W/m^3)')
ax.set_title('Templates (Jmag={:.2f})'.format(jmag))
ax.set_xlim([wmin,wmax])
ax.grid()
fig.tight_layout()
fig.savefig(plot_out,dpi=200,pad_inches=0.02)
plt.close(fig)

