import os
import numpy as np
import argparse

import params
import utils
templates_dir='templates'

aj_over_ak=3.02  # Nishiyama+06 (ApJ, 638, 839)

wave_j=12500.

fh=open(templates_dir+'/list.txt')
list_objs=[]
Jmags={}; sptypes={}
for line in fh.readlines():
    obj, sptype, Jmag_str=line.split()
    list_objs.append(obj)
    Jmags[obj]=float(Jmag_str)
    sptypes[obj]=sptype
fh.close()

def list_templates():
    print('--------------------------')
    print('{0:10s}  {1:6s}  {2}'.format('Name','SpType','Jmag'))
    print('--------------------------')
    for obj in list_objs:
        print('{0:10s}  {1:6s}  {2:6.3f}'.format(obj.replace(' ','_'),sptypes[obj],Jmags[obj]))

def print_spec(waves, fluxs, header=False):
    if header is True:
        print('wavelength,flux')
    for w, f in zip(waves,fluxs):
        print('{0:.2f},{1:.4e}'.format(w,f))


parser=argparse.ArgumentParser(description='load_template.py')
parser.add_argument('-l','--list',action='store_true',help='List available templates')
parser.add_argument('-m','--mode',type=str,default='WIDE',help='Mode (WIDE, HIRES-Y, HIRES-J)')
parser.add_argument('-n','--name',type=str,default=params.option_defaults['name'],help='Name of template')
parser.add_argument('-j','--jmag',type=float,default=params.option_defaults['jmag'],help='J-band magnitude')
parser.add_argument('-s','--step',type=float,default=params.option_defaults['step'],help='Wavelength step (AA)')
parser.add_argument('-ak','--extinction_k',type=float,default=params.option_defaults['ak'],help='K-band extinction (mag)')
parser.add_argument('-aj','--extinction_j',type=float,default=params.option_defaults['aj'],help='J-band extinction (mag)')
parser.add_argument(params.option_short['header'],'--header',action='store_true',default=params.option_defaults['header'],help=params.option_helps['header'])

args=parser.parse_args()

if args.list is True:
    list_templates()
    exit()
mode=args.mode
if mode not in params.list_mode:
    print('Unexpected mode={0}'.format(mode))
    exit()
wmin, wmax=params.list_wranges[mode]
step=args.step
if np.isnan(step):
    print('No wavelength step was given')
    exit()
waves_new=np.arange(wmin,wmax,step)

name=args.name
if name is None:
    print('No name was given')
    exit()
jmag=args.jmag
if np.isnan(jmag):
    jmag=Jmags[name]
aj=args.extinction_j
if (np.isnan(aj)) and (not np.isnan(args.extinction_k)):
    aj=args.extinction_k*aj_over_ak

flux_factor=pow(10.,-0.4*(jmag-Jmags[name]))

file_in='{0}/XSL_{1}.txt'.format(templates_dir,name.replace(' ','_'))
if not os.path.isfile(file_in):
    print('Failed to find {0}'.format(file_in))
    exit()
waves, fluxs=utils.load_template(file_in,wmin,wmax,flux_factor=flux_factor)
fluxs_new=[]
for w in waves_new:
    w0, w1=w-step/2., w+step/2.
    condition=(waves>=w0) & (waves<=w1)
    if np.sum(condition)<=0:
        fluxs_new.append(np.nan)
    else:
        fluxs_new.append(np.mean(fluxs[condition]))
if (not np.isnan(aj)) and (aj>0):
    fluxs=utils.apply_extinction(waves_new,fluxs_new,aj)
if args.header:
    print_spec(waves_new,fluxs_new,header=True)
else:
    print_spec(waves_new,fluxs_new,header=False)

